if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_GcdBar requires PitBull4")
end
local L = PitBull4.L

local EXAMPLE_VALUE  = 0.3

local PitBull4_GcdBar = PitBull4:NewModule("GcdSpark", "AceEvent-3.0")

PitBull4_GcdBar:SetModuleType("bar")
PitBull4_GcdBar:SetName("GCD Bar")
PitBull4_GcdBar:SetDescription("Show the gcd.")
PitBull4_GcdBar:SetDefaults({
	size = 1,
	position = 10,
},{
	gcd_color_foreground   = { 0.75, 0.75, 0.75 },
	gcd_color_background = { 0.25, 0.25, 0.25},
})

local timerFrame = CreateFrame("Frame")
timerFrame:Hide()
timerFrame:SetScript("OnUpdate", function()
	PitBull4_GcdBar:UpdateForUnitID("player")
end)

local last_spellcast = 0

function PitBull4_GcdBar:OnEnable()
	timerFrame:Show()
	
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
end

function PitBull4_GcdBar:OnDisable()
	timerFrame:Hide()
end

function PitBull4_GcdBar:GetValue(frame)
	if frame.unit ~= "player"  then
		return nil
	end
	
	local haste = 0
	if UnitPowerType(frame.unit) == 0 then -- mana
		haste = GetCombatRatingBonus(CR_HASTE_SPELL)
	elseif UnitPowerType(frame.unit) == 2 then -- focus
		haste = GetCombatRatingBonus(CR_HASTE_RANGED)
	else -- melee
		haste = GetCombatRatingBonus(CR_HASTE_MELEE)
	end
	
	haste = 1 + (haste/100)
	
	local gcd_time = 1.5 / haste
	if (gcd_time < 1) then
		gcd_time = 1
	end
	
	local time_since_spellcast = GetTime() - last_spellcast
	if time_since_spellcast > gcd_time then
		return 0
	end
	
	return time_since_spellcast / gcd_time
end

function PitBull4_GcdBar:GetExampleValue(frame)
	if frame and frame.unit ~= "player" then
		return nil
	end
	return EXAMPLE_VALUE
end

function PitBull4_GcdBar:GetColor(frame, value)
	local r, g, b = unpack(self.db.profile.global.gcd_color_foreground)
	return r, g, b, 1
end

function PitBull4_GcdBar:GetBackgroundColor(frame, value)
	local r, g, b = unpack(self.db.profile.global.gcd_color_background)
	return r, g, b, 1
end

function PitBull4_GcdBar:UNIT_SPELLCAST_START(event, unit, spell)
	if unit ~= 'player' then
		return
	end
	
	local start, dur = GetSpellCooldown(spell)
	if dur and dur > 0 and dur <= 1.5 then
		last_spellcast = start
	end
end

PitBull4_GcdBar.UNIT_SPELLCAST_CHANNEL_START = PitBull4_GcdBar.UNIT_SPELLCAST_START
PitBull4_GcdBar.UNIT_SPELLCAST_SUCCEEDED = PitBull4_GcdBar.UNIT_SPELLCAST_START
PitBull4_GcdBar:SetLayoutOptionsFunction(function(self) end)

PitBull4_GcdBar:SetColorOptionsFunction(function(self)
	return 'GCD', {
		type = 'group',
		name = L["GCD"],
		inline = true,
		args = {
			gcd_color_foreground = {
				type = 'color',
				name = L['GCD Foreground'],
				desc = L['Sets which color to use on the gcd bar'],
				get = function(info)
					return unpack(self.db.profile.global.gcd_color_foreground)
				end,
				set = function(info, r, g, b)
					self.db.profile.global.gcd_color_foreground = { r, g, b }
					self:UpdateAll()
				end,
				order = 1,
			},
			gcd_color_background = {
				type = 'color',
				name = L['GCD Foreground'],
				desc = L['Sets which color to use behind the gcd bar'],
				get = function(info)
					return unpack(self.db.profile.global.gcd_color_background)
				end,
				set = function(info, r, g, b)
					self.db.profile.global.gcd_color_background = { r, g, b }
					self:UpdateAll()
				end,
				order = 1,
			},
		},
	},
	function(info)
		self.db.profile.global.gcd_color_foreground = { 0.25, 0.25, 0.25 }
		self.db.profile.global.gcd_color_background = { 0.75, 0.75, 0.75 }
	end
end)
